"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mqttHandler = exports.initDeviceService = exports.deviceFeatures = void 0;
const hubDevice_1 = require("./hubDevice");
const motionDevice_1 = require("./motionDevice");
const leakDevice_1 = require("./leakDevice");
const valveDevice_1 = require("./valveDevice");
const thermoHydroDevice_1 = require("./thermoHydroDevice");
const contactDevice_1 = require("./contactDevice");
const switchDevice_1 = require("./switchDevice");
const outletDevice_1 = require("./outletDevice");
const statelessSwitch_1 = require("./statelessSwitch");
const garageDoor_1 = require("./garageDoor");
const lockDevice_1 = require("./lockDevice");
exports.deviceFeatures = {
    Hub: { experimental: false, hasBattery: false },
    SpeakerHub: { experimental: false, hasBattery: false },
    VibrationSensor: { experimental: false, hasBattery: true },
    MotionSensor: { experimental: false, hasBattery: true },
    LeakSensor: { experimental: false, hasBattery: true },
    Manipulator: { experimental: false, hasBattery: true },
    THSensor: { experimental: false, hasBattery: true },
    DoorSensor: { experimental: false, hasBattery: true },
    Siren: { experimental: false, hasBattery: true },
    Switch: { experimental: false, hasBattery: false },
    Outlet: { experimental: false, hasBattery: false },
    SmartRemoter: { experimental: false, hasBattery: true },
    MultiOutlet: { experimental: false, hasBattery: false },
    GarageDoor: { experimental: false, hasBattery: false },
    Finger: { experimental: false, hasBattery: true },
    GarageDoorCombo: { experimental: false, hasBattery: false },
    Lock: { experimental: true, hasBattery: true },
};
exports.initDeviceService = {
    Hub() { hubDevice_1.initHubDevice.bind(this)(); },
    SpeakerHub() { hubDevice_1.initHubDevice.bind(this)(); },
    VibrationSensor() { motionDevice_1.initMotionSensor.bind(this)(); },
    MotionSensor() { motionDevice_1.initMotionSensor.bind(this)(); },
    LeakSensor() { leakDevice_1.initLeakSensor.bind(this)(); },
    Manipulator() { valveDevice_1.initValveDevice.bind(this)(); },
    THSensor() { thermoHydroDevice_1.initThermoHydroDevice.bind(this)(); },
    DoorSensor() { contactDevice_1.initContactSensor.bind(this)(); },
    Siren() { switchDevice_1.initSwitchDevice.bind(this)('alert', { 'alarm': true }, { 'alarm': false }); },
    Switch() { switchDevice_1.initSwitchDevice.bind(this)('open', 'open', 'close'); },
    Outlet() { outletDevice_1.initOutletDevice.bind(this)('open', 'open', 'close'); },
    SmartRemoter() { statelessSwitch_1.initStatelessSwitch.bind(this, 4)(); },
    MultiOutlet() { outletDevice_1.initOutletDevice.bind(this)('open', 'open', 'close'); },
    GarageDoor() { switchDevice_1.initSwitchDevice.bind(this)('toggle', '', ''); },
    Finger() { switchDevice_1.initSwitchDevice.bind(this)('toggle', '', ''); },
    GarageDoorCombo() { garageDoor_1.initGarageDoor.bind(this)(); },
    Lock() { lockDevice_1.initLockDevice.bind(this)(); },
};
exports.mqttHandler = {
    Hub(data) { hubDevice_1.mqttHubDevice.bind(this)(data); },
    SpeakerHub(data) { hubDevice_1.mqttHubDevice.bind(this)(data); },
    VibrationSensor(data) { motionDevice_1.mqttMotionSensor.bind(this)(data); },
    MotionSensor(data) { motionDevice_1.mqttMotionSensor.bind(this)(data); },
    LeakSensor(data) { leakDevice_1.mqttLeakSensor.bind(this)(data); },
    Manipulator(data) { valveDevice_1.mqttValveDevice.bind(this)(data); },
    THSensor(data) { thermoHydroDevice_1.mqttThermoHydroDevice.bind(this)(data); },
    DoorSensor(data) { contactDevice_1.mqttContactSensor.bind(this)(data); },
    Siren(data) { switchDevice_1.mqttSwitchDevice.bind(this)(data); },
    Switch(data) { switchDevice_1.mqttSwitchDevice.bind(this)(data); },
    Outlet(data) { outletDevice_1.mqttOutletDevice.bind(this)(data); },
    SmartRemoter(data) { statelessSwitch_1.mqttStatelessSwitch.bind(this)(data); },
    MultiOutlet(data) { outletDevice_1.mqttOutletDevice.bind(this)(data); },
    GarageDoor(data) { switchDevice_1.mqttSwitchDevice.bind(this)(data); },
    Finger(data) { switchDevice_1.mqttSwitchDevice.bind(this)(data); },
    GarageDoorCombo(data) { garageDoor_1.mqttGarageDoor.bind(this)(data); },
    Lock(data) { lockDevice_1.mqttLockDevice.bind(this)(data); },
};
//# sourceMappingURL=deviceHandlers.js.map