/***********************************************************************
 * YoLink Platform Accessory class
 *
 * Copyright (c) 2022 David Kerr
 *
 * Based on https://github.com/homebridge/homebridge-plugin-template
 *
 * An instance of this class is created for each accessory the platform registers.
 *
 */
import { PlatformAccessory, CharacteristicValue } from 'homebridge';
import { YoLinkHomebridgePlatform } from './platform';
export declare class YoLinkPlatformAccessory {
    readonly platform: YoLinkHomebridgePlatform;
    readonly accessory: PlatformAccessory;
    config: {
        [key: string]: any;
    };
    [key: string]: any;
    deviceId: any;
    deviceMsgName: any;
    constructor(platform: YoLinkHomebridgePlatform, accessory: PlatformAccessory);
    /*********************************************************************
     * checkDeviceState
     *
     */
    initializeDeviceVars(platform: any, device: any): void;
    /*********************************************************************
     * checkDeviceState
     * Updates device status object, sending a request to the YoLink API if we
     * have no data yet, or it has been a long time since the data was updated.
     *
     * Calls to this function should be serialized with deviceSemaphore to
     * prevent sending multiple requests for the same data to the server.
     */
    checkDeviceState(platform: any, device: any): Promise<any>;
    /***********************************************************************
     * logDeviceState
     *
     */
    logDeviceState(this: YoLinkPlatformAccessory, device: any, msg: string): void;
    /*********************************************************************
     * refreshDataTimer
     * We run a timer for each device to update cached data from the YoLink servers.
     * Timer is set such that it will fire when the updateTime arrives and (via the
     * provided callback) will call above checkDeviceState function at just the
     * right time to force call to yolinkAPI.getDeviceState.  All this to optimize
     * performance of user experience.
     */
    refreshDataTimer(handleGet: any): Promise<void>;
    /***********************************************************************
     * updateBatteryInfo
     *
     */
    updateBatteryInfo(this: YoLinkPlatformAccessory, device: any): number;
    /*********************************************************************
     * handleBatteryGet
     *
     */
    handleBatteryGet(this: YoLinkPlatformAccessory, device: any): Promise<CharacteristicValue>;
    /*********************************************************************
     * handleIdentifySet
     *
     */
    handleIdentifySet(this: YoLinkPlatformAccessory, device: any, value: any): Promise<void>;
    /*********************************************************************
     * mqttMessage
     *
     */
    mqttMessage(message: any): Promise<void>;
}
//# sourceMappingURL=platformAccessory.d.ts.map