/***********************************************************************
 * YoLink smart remote device support (as a HomeKit stateless switch)
 *
 * Copyright (c) 2022 David Kerr
 *
 */
import { YoLinkPlatformAccessory } from './platformAccessory';
/***********************************************************************
 * initStatelessSwitch
 *
 */
export declare function initStatelessSwitch(this: YoLinkPlatformAccessory, nButtons: number): Promise<void>;
/***********************************************************************
 * mqttStatelessSwitch
 *
 * Example of message received,
 *
 * {
 *   "event":"SmartRemoter.Report",
 *   "time":1660337968343,
 *   "msgid":"1660337968342",
 *   "data":{
 *     "event":{
 *       "keyMask":0,
 *       "type":"LongPress"
 *     },
 *     "battery":4,
 *     "version":"0406",
 *     "devTemperature":30,
 *     "loraInfo":{
 *       "signal":-77,
 *       "gatewayId":"abcdef1234567890",
 *       "gateways":1
 *     }
 *   },
 *   "deviceId":"abcdef1234567890"
 * }
 *
 * {
 *   "event":"SmartRemoter.StatusChange",
 *   "time":1660334748994,
 *   "msgid":"1660334748990",
 *   "data":{
 *     "event":{
 *       "keyMask":1,
 *       "type":"Press"
 *     },
 *     "battery":4,
 *     "version":"0406",
 *     "devTemperature":30,
 *     "loraInfo":{
 *       "signal":-85,
 *       "gatewayId":"abcdef1234567890",
 *       "gateways":1
 *     }
 *   },
 *   "deviceId":"abcdef1234567890"
 * }
 *
 * "keyMask" is a bit field. E.g. for a four button remote the bits set will be 1, 2, 4, 8.  If you press
 * two buttons simultaneously then you will get e.g. 9 for buttons one and four... as a "LongPress".
 *
 * "type" can be "Press" or "LongPress"
 */
export declare function mqttStatelessSwitch(this: YoLinkPlatformAccessory, message: any): Promise<void>;
//# sourceMappingURL=statelessSwitch.d.ts.map