"use strict";
/***********************************************************************
 * YoLink unknown device support
 *
 * This will issue warning messages to the HomeBridge log that can then
 * be provided to author to assist in adding device support.
 *
 * Copyright (c) 2022 David Kerr
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.mqttUnknownDevice = exports.initUnknownDevice = void 0;
/***********************************************************************
 * initUnknownDevice
 *
 */
async function initUnknownDevice() {
    const platform = this.platform;
    const device = this.accessory.context.device;
    platform.log.warn(`YoLink device type: '${device.type}' is not supported (${device.deviceMsgName}) (initialize)`
        + platform.reportError + JSON.stringify(device));
    this.refreshDataTimer(handleGet.bind(this));
}
exports.initUnknownDevice = initUnknownDevice;
/***********************************************************************
 * handleGet
 *
 */
async function handleGet() {
    const platform = this.platform;
    const device = this.accessory.context.device;
    // serialize access to device data.
    const releaseSemaphore = await device.semaphore.acquire();
    try {
        if (await this.checkDeviceState(platform, device)) {
            platform.log.warn(`YoLink device type: '${device.type}' is not supported (${device.deviceMsgName}) (handleGet)`
                + platform.reportError + JSON.stringify(device.data));
        }
        else {
            platform.log.error(`Device offline or other error for ${device.deviceMsgName}`);
        }
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in UnknownDevice handleGet' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
    return (false);
}
/***********************************************************************
 * mqttUnknownDevice
 *
 */
async function mqttUnknownDevice(message) {
    const platform = this.platform;
    const device = this.accessory.context.device;
    // serialize access to device data.
    const releaseSemaphore = await device.semaphore.acquire();
    try {
        platform.log.warn(`YoLink device type: '${device.type}' is not supported (${device.deviceMsgName}) (MQTT)`
            + platform.reportError + JSON.stringify(message));
    }
    catch (e) {
        const msg = (e instanceof Error) ? e.stack : e;
        platform.log.error('Error in mqttUnknownDevice' + platform.reportError + msg);
    }
    finally {
        await releaseSemaphore();
    }
}
exports.mqttUnknownDevice = mqttUnknownDevice;
//# sourceMappingURL=unknownDevice.js.map