/***********************************************************************
 * YoLink API class
 *
 * Copyright (c) 2022 David Kerr
 *
 * Based on documentation at http://doc.yosmart.com
 *
 * Manages login with User Access Credentials (UAC), obtaining list
 * of devices from YoLink servers, getting and setting values, and
 * subscribing to alerts and messages with MQTT
 */
import { YoLinkHomebridgePlatform } from './platform';
declare type yolinkBUDP = {
    time: number;
    method: string;
    msgid: string;
    code: string;
    desc: string;
    data: {
        [key: string]: any;
    };
    [key: string]: any;
};
/***********************************************************************
 * YoLinkAPI class and constructor
 *
 */
export declare class YoLinkAPI {
    private readonly platform;
    private yolinkTokens;
    private yolinkHomeId;
    private yolinkLoggedIn;
    private accessSemaphore;
    private accessTokenRefreshAt;
    private accesstokenHeartbeatAt;
    private accessTokenExpireTime;
    private accessTokenHeartbeat;
    private mqttTokenExpireTime;
    private mqttHost;
    private mqttClient;
    constructor(platform: YoLinkHomebridgePlatform);
    /*********************************************************************
     * login
     *
     */
    login(platform: YoLinkHomebridgePlatform): Promise<void>;
    tryLogin(platform: YoLinkHomebridgePlatform): Promise<void>;
    /*********************************************************************
     * getAccessToken
     *
     */
    getAccessToken(platform: YoLinkHomebridgePlatform): Promise<string>;
    tryGetAccessToken(platform: YoLinkHomebridgePlatform): Promise<string>;
    /*********************************************************************
     * getDeviceList
     *
     */
    getDeviceList(platform: YoLinkHomebridgePlatform): Promise<unknown>;
    tryGetDeviceList(platform: YoLinkHomebridgePlatform): Promise<any>;
    /*********************************************************************
     * getDeviceState
     *
     */
    getDeviceState(platform: YoLinkHomebridgePlatform, device: any): Promise<yolinkBUDP>;
    tryGetDeviceState(platform: YoLinkHomebridgePlatform, device: any): Promise<yolinkBUDP>;
    /*********************************************************************
     * setDeviceState
     *
     */
    setDeviceState(platform: YoLinkHomebridgePlatform, device: any, state: any, method?: string): Promise<yolinkBUDP>;
    trySetDeviceState(platform: YoLinkHomebridgePlatform, device: any, state: any, method?: string): Promise<yolinkBUDP>;
    /*********************************************************************
     * mqtt
     *
     * Open a MQTT session with YoLink API server to receive update
     * messages from YoLink for each device.
     *
     * This has not been thoroughly tested.  How it behaves on roaming (IP
     * address changes) or temporary disconnects/reconnects has not been
     * full tested.  Info logging is enabled to capture as much information
     * as possible for events like this. Please report bugs.
     */
    mqtt(platform: YoLinkHomebridgePlatform, msgCallback: any): void;
}
export {};
//# sourceMappingURL=yolinkAPI.d.ts.map